#import <Foundation/Foundation.h>
#import "MRConfig.h"
@class MRLogMessage;
@protocol MRLogger;

NS_ASSUME_NONNULL_BEGIN

#pragma mark - MRLog
@interface MRLog : NSObject
+ (NSArray *)loggers;
+ (NSArray *)loggersOfClass:(Class)c;
+ (void)setLoggers:(NSArray <id<MRLogger>> *)loggers;
+ (void)addLogger:(id<MRLogger>)logger;
+ (void)removeLogger:(id<MRLogger>)logger;
+ (void)log:(NSString *)formatString, ...;
+ (void)logMessage:(nullable MRLogMessage *)message;
+ (MRLogLevel)logLevel;
+ (void)setLogLevel:(MRLogLevel)logLevel;
+ (MRLogLevel)logLevelForDomain:(NSString *)domain;
+ (void)setLogLevel:(MRLogLevel)level forDomain:(NSString *)domain;

+ (void)logWithLevel:(MRLogLevel)level
           timestamp:(NSDate *)timestamp
              domain:(nullable NSString *)domain
            userInfo:(nullable NSDictionary *)userInfo
                file:(const char *)file
            function:(const char *)function
                line:(NSUInteger)line
              format:(nullable NSString *)format, ... NS_FORMAT_FUNCTION(8, 9);

@end

#pragma mark - MRLogMessage
@interface MRLogMessage : NSObject <NSSecureCoding>
- (id)initWithMessage:(nullable NSString *)message
    level:(MRLogLevel)level
    timestamp:(NSDate *)timestamp
    domain:(nullable NSString *)domain
    userInfo:(nullable NSDictionary *)userInfo
    file:(const char *)file
    function:(const char *)function
    line:(NSInteger)line;

@property (nonatomic, readonly, copy) NSString *message;
@property (nonatomic, readonly) MRLogLevel level;
@property (nonatomic, readonly, copy) NSDate *timestamp;
@property (nonatomic, readonly, copy) NSString *domain;
@property (nonatomic, readonly, copy) NSDictionary *userInfo;
@property (nonatomic, readonly, copy) NSString *file;
@property (nonatomic, readonly, copy) NSString *function;
@property (nonatomic, readonly) NSInteger line;
@end

#pragma mark - MRLogFormatter
@interface MRLogFormatter : NSObject
/// Format a log message. Uses log level to determine format
/// @param message @c MRLogMessage to format.
/// @return @c NSString formatted log message
- (NSString *)formatLogMessage:(MRLogMessage *)message;

/// Format an array of log messages
/// @param messages array of @c MRLogMessage objects
/// @return array of formatted messages
- (NSString *)formatLogMessages:(NSArray *)messages;
@end

@interface MREmailLogFormatter : MRLogFormatter
@end

@interface MRTTYLogFormatter : MRLogFormatter
@end

#pragma mark - MRLogger
@protocol MRLogger <NSObject>
/// Log level
@property (nonatomic) MRLogLevel logLevel;
/// Log formatter
@property (nonatomic, strong) MRLogFormatter *formatter;
/// Log an @c MRLogMessage
- (void)logMessage:(MRLogMessage *)message;
@end

@interface MRTTYLogger : NSObject <MRLogger>
@end

@interface MRCampaignLogger : NSObject <MRLogger>
/// All the messages for a the given Major/Minor string
- (NSArray *)messagesForMajorMinors:(NSArray *)majorMinors;

/// Load all the available archived logs
/// @return @c YES if loading was successful, @c NO if something went wrong
- (BOOL)loadLogsFromDisk;

/// Archive the current set of logs off to disk.
/// @return @c YES if successful, @c NO if something went wrong
- (BOOL)archiveLogsToDisk;

/// Writes the current set of logs off to disk.
/// @return @c YES if successful, @c NO if something went wrong
- (NSURL *)writeFormattedLogsToTempFile;
@end


#pragma mark -
extern NSString * const MRDefaultLoggerDomain;
extern NSString * const MRCampaignLoggerDomain;
extern NSString * const MRCampaignLoggerMajorMinorKey;
extern NSString * const MRCampaignLoggerMessageLoggedNotification;

NS_ASSUME_NONNULL_END

#define MR_LOG_MACRO(lvl, time, dom, func, fmt, ...) \
    [MRLog logWithLevel : lvl                        \
              timestamp : time                       \
                 domain : dom                        \
               userInfo : nil                        \
                   file : __FILE__                   \
               function : func                       \
                   line : __LINE__                   \
                format : (fmt), ##__VA_ARGS__]


#define MRDomainLogError(domain, fmt, ...)    MR_LOG_MACRO(MRLogLevelError,   [NSDate date], domain, __PRETTY_FUNCTION__, fmt, ##__VA_ARGS__)
#define MRDomainLogWarning(domain, fmt, ...)  MR_LOG_MACRO(MRLogLevelWarn,    [NSDate date], domain, __PRETTY_FUNCTION__, fmt, ##__VA_ARGS__)
#define MRDomainLogInfo(domain, fmt, ...)     MR_LOG_MACRO(MRLogLevelInfo,    [NSDate date], domain, __PRETTY_FUNCTION__, fmt, ##__VA_ARGS__)
#define MRDomainLogDebug(domain, fmt, ...)    MR_LOG_MACRO(MRLogLevelDebug,   [NSDate date], domain, __PRETTY_FUNCTION__, fmt, ##__VA_ARGS__)
#define MRDomainLogVerbose(domain, fmt, ...)  MR_LOG_MACRO(MRLogLevelVerbose, [NSDate date], domain, __PRETTY_FUNCTION__, fmt, ##__VA_ARGS__)

#define MRLogError(fmt, ...)    MRDomainLogError(MRDefaultLoggerDomain,   fmt, ##__VA_ARGS__)
#define MRLogWarning(fmt, ...)  MRDomainLogWarning(MRDefaultLoggerDomain, fmt, ##__VA_ARGS__)
#define MRLogInfo(fmt, ...)     MRDomainLogInfo(MRDefaultLoggerDomain,    fmt, ##__VA_ARGS__)
#define MRLogDebug(fmt, ...)    MRDomainLogDebug(MRDefaultLoggerDomain,   fmt, ##__VA_ARGS__)
#define MRLogVerbose(fmt, ...)  MRDomainLogVerbose(MRDefaultLoggerDomain, fmt, ##__VA_ARGS__)

#define MRLogf(fmt, ...)    [MRLog logMessage:[[MRLogMessage alloc] initWithMessage:fmt!=nil?[NSString stringWithFormat:fmt, ##__VA_ARGS__]:nil level:MRLogLevelNormal timestamp:[NSDate date] domain:nil userInfo:nil file:__FILE__ function:__PRETTY_FUNCTION__ line:__LINE__]]
#define MRDebugf(fmt, ...)  [MRLog logMessage:[[MRLogMessage alloc] initWithMessage:fmt!=nil?[NSString stringWithFormat:fmt, ##__VA_ARGS__]:nil level:MRLogLevelDebug timestamp:[NSDate date] domain:nil userInfo:nil file:__FILE__ function:__PRETTY_FUNCTION__ line:__LINE__]]

#define MRCampaignLogf(majorminor, fmt, ...)    [MRLog logMessage:[[MRLogMessage alloc] initWithMessage:fmt!=nil?[NSString stringWithFormat:fmt, ##__VA_ARGS__]:nil level:MRLogLevelNormal timestamp:[NSDate date] domain:MRCampaignLoggerDomain userInfo:majorminor?@{MRCampaignLoggerMajorMinorKey:(id)majorminor}:nil file:__FILE__ function:__PRETTY_FUNCTION__ line:__LINE__]]
#define MRCampaignDebugf(majorminor, fmt, ...)  [MRLog logMessage:[[MRLogMessage alloc] initWithMessage:fmt!=nil?[NSString stringWithFormat:fmt, ##__VA_ARGS__]:nil level:MRLogLevelDebug timestamp:[NSDate date] domain:MRCampaignLoggerDomain userInfo:majorminor?@{MRCampaignLoggerMajorMinorKey:(id)majorminor}:nil file:__FILE__ function:__PRETTY_FUNCTION__ line:__LINE__]]
