//
//  MSCustomRouteViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/22/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSCustomRouteViewController: UIViewController, MRMapViewDelegate, MSExampleViewController {
    private lazy var mapView: MRMapView = {
        let mapView = MRMapView()
        mapView.delegate = self
        mapView.showsUserLocation = true
        mapView.showsRoutePath = false
        mapView.mapKey = MREditorKey(forMap:MAP_ID, app:APP_ID)
        return mapView
    }()

    private var directions: MRDirections?
    private var routeOverlay: MRPathOverlay?

    override func viewDidLoad() {
        super.viewDidLoad()
        self.view = self.mapView

        let button0 = UIBarButtonItem(title: "Directions", style: .plain, target: self, action: #selector(self.directionsAction))
        self.navigationItem.rightBarButtonItems = [button0]
    }

    // MARK: - MSExampleViewController

    static func exampleTitle() -> String {
        return "Custom Route Line"
    }

    static func exampleDetail() -> String {
        return "Example of how to customize the route path.\n\n" +
            "• `Directions` button to start and stop directions from the selected placemark.\n" +
            "• The default route path is now a red line of width 2.\n"
    }

    // MARK: - MRMapViewDelegate
    
    func mapPickerDidPick(_ map: MRMap) {
        self.mapView.mapKey = map.key
    }
    
    func viewControllerToAddSheetTo(for mapView: MRMapView) -> UIViewController {
        return self
    }
    
    func mapView(_ mapView: MRMapView, rendererFor overlay: MRPathOverlay) -> MRPathRenderer {
        let renderer = MRPathRenderer(overlay: overlay)
        renderer.strokeColor = UIColor.red
        renderer.lineCap = .round
        renderer.lineJoin = .round
        renderer.lineWidth = 2
        return renderer
    }

    func mapViewDidFinishLoadingMap(_ mapView: MRMapView) {
        self.reloadRouteOverlay()
    }

    // MARK: - Actions

    @objc private func directionsAction() {
        if self.mapView.route != nil {
            self.directions?.cancel()
            self.directions = nil
            self.mapView.setRoute(nil, animated:true)
            self.reloadRouteOverlay()
            return
        }

        guard let appKey = self.mapView.mapKey.parent else {
            return
        }
        guard let destinationAnnotation = self.mapView.selectedAnnotation as? MRPlacemark else {
            return
        }
        let request = MRDirectionsRequest()
        request.app = appKey
        request.destination = MRDirectionsDestination(placemarkKey: destinationAnnotation.key)
        request.source = MRDirectionsSource.withCurrentLocation()
        self.directions = MRDirections(request: request, presenting: self)
        self.directions?.calculate(completionHandler: { [weak self] (response, error) in
            self?.directionsResponseDidLoad(response: response, error: error)
        })
    }

    // MARK: - Internal

    private func directionsResponseDidLoad(response: MRDirectionsResponse?, error: Error?) {
        self.directions = nil
        if let routeResponse = response?.routes, error == nil, routeResponse.count > 0 {
            self.mapView.deselectAnnotation(animated: false)
            self.mapView.setRoute(routeResponse[0], animated: true)
            self.reloadRouteOverlay()
            return
        }

        // Error or no routes:
        let alertController = UIAlertController(title: "Could not load directions", message: error?.localizedDescription, preferredStyle: .alert)
        let OKAction = UIAlertAction(title: "OK", style: .default)
        alertController.addAction(OKAction)
        self.present(alertController, animated: true)
    }

    private func reloadRouteOverlay() {
        if let overlay = self.routeOverlay {
            self.mapView.remove(overlay)
            self.routeOverlay = nil
        }
        if let route = self.mapView.route, let steps = route.steps {
            let path = UIBezierPath()
            for step in steps {
                if step.mapKey == self.mapView.mapKey {
                    path.append(step.path)
                }
            }
            self.routeOverlay = MRPathOverlay(path: path.cgPath)
            self.mapView.add(routeOverlay!)
        }
    }
}
