//
//  MSDirectionsViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/22/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSDirectionsViewController: UIViewController, MRMapViewDelegate, MSExampleViewController {
    private lazy var mapView: MRMapView = {
        let mapView = MRMapView()
        mapView.delegate = self
        mapView.showsUserLocation = true
        mapView.mapKey = MREditorKey(forMap:MAP_ID, app:APP_ID)
        return mapView
    }()

    private var directions: MRDirections?
    private var directionsVisible = true

    override func viewDidLoad() {
        super.viewDidLoad()
        self.view = self.mapView

        let button0 = UIBarButtonItem(title: "Overview", style: .plain, target: self, action: #selector(self.overviewAction))
        let button1 = UIBarButtonItem(title: "Hide", style: .plain, target: self, action: #selector(self.hideAction))
        let button2 = UIBarButtonItem(title: "Last", style: .plain, target: self, action: #selector(self.lastStepAction))
        let button3 = UIBarButtonItem(title: "DirTo:", style: .plain, target: self, action: #selector(self.directionsToAction))
        let button4 = UIBarButtonItem(title: "DirFrom", style: .plain, target: self, action: #selector(self.directionsFromAction))
        self.navigationItem.rightBarButtonItems = [button0, button1, button2, button3, button4]
    }
    
    // MARK: - MRMapViewDelegate
    
    func mapPickerDidPick(_ map: MRMap) {
        self.mapView.mapKey = map.key
    }
    
    func viewControllerToAddSheetTo(for mapView: MRMapView) -> UIViewController {
        return self
    }

    // MARK: - MSExampleViewController

    static func exampleTitle() -> String {
        return "Directions"
    }

    static func exampleDetail() -> String {
        return "Example of various route API usage.\n\n" +
            "• `DirTo.` button to start and stop directions to the selected placemark.\n" +
            "• `DirFrom.` button to start and stop directions from the selected placemark.\n" +
            "• `Last.` button to jump to the last step.\n" +
            "• `Hide.` button to show/hide the directions control.\n" +
            "• `Overview` button to scroll to the full route path.\n"
    }

    // MARK: - Action

    @objc private func overviewAction() {
        self.mapView.scrollToOverview()
    }

    @objc private func hideAction() {
        self.mapView.setDirectionsControlVisible(self.directionsVisible)
        self.directionsVisible = !self.directionsVisible
    }

    @objc private func lastStepAction() {
        if let stepCount = self.mapView.route?.steps?.count {
            self.mapView.setRouteStep(UInt(stepCount - 1), animated:true)
        }
    }

    @objc private func directionsToAction() {
        if self.mapView.route != nil {
            self.mapView.setRoute(nil, animated:true)
            return
        }

        guard let appKey = self.mapView.mapKey.parent else {
            return
        }
        guard let destinationAnnotation = self.mapView.selectedAnnotation as? MRPlacemark else {
            return
        }
        let request = MRDirectionsRequest()
        request.app = appKey
        request.destination = MRDirectionsDestination(placemarkKey: destinationAnnotation.key)
        request.source = MRDirectionsSource.withCurrentLocation()
        self.directions = MRDirections(request: request, presenting: self)
        self.directions?.calculate(completionHandler: {[weak self] (response, error) in
            self?.directionsResponseDidLoad(response: response, error: error)
        })
    }

    @objc private func directionsFromAction() {
        if self.mapView.route != nil {
            self.mapView.setRoute(nil, animated:true)
            return
        }

        guard let appKey = self.mapView.mapKey.parent else {
            return
        }
        guard let sourceAnnotation = self.mapView.selectedAnnotation as? MRPlacemark else {
            return
        }
        let request = MRDirectionsRequest()
        request.app = appKey
        request.destination = MRDirectionsDestination.withCurrentLocation()
        request.source = MRDirectionsSource(placemarkKey: sourceAnnotation.key)
        self.directions = MRDirections(request: request, presenting: self)
        self.directions?.calculate(completionHandler: {[weak self] (response, error) in
            self?.directionsResponseDidLoad(response: response, error: error)
        })
    }

    // MARK: - Internal

    private func directionsResponseDidLoad(response: MRDirectionsResponse?, error: Error?) {
        self.directions = nil
        if let routeResponse = response?.routes, error == nil, routeResponse.count > 0 {
            self.mapView.deselectAnnotation(animated: false)
            self.mapView.setRoute(routeResponse[0], animated: true)
            return
        }

        // Error or no routes:
        let alertController = UIAlertController(title: "Could not load directions", message: error?.localizedDescription, preferredStyle: .alert)
        let OKAction = UIAlertAction(title: "OK", style: .default)
        alertController.addAction(OKAction)
        self.present(alertController, animated: true)
    }
}
