//
//  MSNearbyTableViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/23/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSNearbyTableViewController: UITableViewController, UISearchBarDelegate, MRLocationManagerDelegate, MSExampleViewController {
    private lazy var searchBar: UISearchBar = {
        let searchBar = UISearchBar(frame: CGRect(x: 0, y: 0, width: self.view.frame.size.width, height: 44))
        searchBar.delegate = self
        searchBar.placeholder = "Find up to ten local items"
        return searchBar
    }()

    private lazy var locationManager: MRLocationManager = {
        let locationManager = MRLocationManager(app: MREditorKey(identifier: APP_ID))
        locationManager.delegate = self
        return locationManager
    }()

    private var search: MRLocalSearch?
    private var autoUpdateTimer: Timer?
    private var searchResults = [MRPlacemarkResult]()

    override func viewDidLoad() {
        super.viewDidLoad()
        self.navigationItem.titleView = self.searchBar
        self.refreshControl?.addTarget(self, action: #selector(self.updateNearby), for: .valueChanged)
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)

        // start listening for location updates
        self.locationManager.startUpdatingLocation()
        self.updateNearby()
    }

    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        self.locationManager.stopUpdatingLocation()
    }

    // MARK: - MSExampleViewController

    static func exampleTitle() -> String {
        return "Local Search"
    }

    static func exampleDetail() -> String {
        return "Example of how to use the MRLocalSearch API to search for Placemarks based on the current location and display the results."
    }

    // MARK: - MRLocationManagerDelegate

    func locationManager(_ manager: MRLocationManager, didUpdateTo location: MRLocation) {
        guard let localSearch = self.search else {
            return
        }
        if self.searchResults.count == 0 && !localSearch.isSearching {
            self.updateNearby()
        }
    }

    // MARK: - UISearchBarDelegate

    func searchBarShouldBeginEditing(_ searchBar: UISearchBar) -> Bool {
        let cancelItem = UIBarButtonItem.init(barButtonSystemItem: .cancel, target: self, action: #selector(self.cancelTapped))
        self.navigationItem.setRightBarButton(cancelItem, animated: true)
        return true
    }

    func searchBarShouldEndEditing(_ searchBar: UISearchBar) -> Bool {
        self.navigationItem.setRightBarButton(nil, animated: true)
        return true
    }

    func searchBarSearchButtonClicked(_ searchBar: UISearchBar) {
        self.updateNearby()
    }

    func searchBar(_ searchBar: UISearchBar, textDidChange searchText: String) {
        if let timer = self.autoUpdateTimer {
            timer.invalidate()
            self.autoUpdateTimer = nil
        }

        // automatically update search results 1 second after you finish typing, even if you didn't hit the Search button
        self.autoUpdateTimer = Timer.scheduledTimer(timeInterval: 1, target: self, selector: #selector(self.updateNearby), userInfo: nil, repeats: false)
    }

    // MARK: - UITableViewController

    override func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
        self.searchBar.resignFirstResponder()
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.searchResults.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let reuseIdentifier = "MRNearbyPlacemarkCell"
        var cell: UITableViewCell
        if let dequeueCell = tableView.dequeueReusableCell(withIdentifier: reuseIdentifier) {
            cell = dequeueCell
        } else {
            cell = UITableViewCell(style: .value1, reuseIdentifier: reuseIdentifier)
        }
        if indexPath.row < searchResults.count {
            let result = self.searchResults[indexPath.row]
            if let placemarkName = result.placemark.name, placemarkName.count > 0 {
                cell.textLabel?.text = placemarkName
            } else if let typeName = result.placemark.typeName, typeName.count > 0 {
                cell.textLabel?.text = typeName
            } else {
                cell.textLabel?.text = "Placemark"
            }
            if result.expectedTravelTime != 0 {
                cell.detailTextLabel?.text = self.timeStringFor(interval: result.expectedTravelTime)
            } else {
                cell.detailTextLabel?.text = ""
            }

            cell.accessoryType = .disclosureIndicator
        }
        return cell
    }

    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if indexPath.row < self.searchResults.count {
            self.presentDirections(to: self.searchResults[indexPath.row].placemark)
        }
    }

    // MARK: - Actions

    @objc private func updateNearby() {
        // a valid location is required for local search
        guard let currentLocation = self.locationManager.location else {
            self.refreshControl?.endRefreshing()
            return
        }

        // kill the auto-update timer if it was running
        self.autoUpdateTimer?.invalidate()
        self.autoUpdateTimer = nil

        self.refreshControl?.beginRefreshing()

        // configure a local search with our current location and a search term if appropriate
        let request = MRLocalSearchRequest()
        request.app = MREditorKey(identifier: APP_ID)
        request.location = currentLocation
        request.transportType = .walking
        request.limit = 10
        request.naturalLanguageQuery = self.searchBar.text

        self.search = MRLocalSearch(request: request)
        self.search?.start(completionHandler: { [weak self] response, error in
            if let lastSearchResults = response?.results {
                guard let strongSelf = self else {
                    return
                }
                strongSelf.searchResults = lastSearchResults
                strongSelf.refreshControl?.endRefreshing()
                DispatchQueue.main.async {
                    strongSelf.tableView.reloadData()
                }
            } else if let lastError = error {
                print("Error loading search results: " + lastError.localizedDescription)
            }
        })
    }

    @objc private func cancelTapped() {
        self.searchBar.text = nil
        self.updateNearby()
        self.searchBar.resignFirstResponder()
    }

    // MARK: - Utility

    private func timeStringFor(interval: TimeInterval) -> String {
        if (interval >= 60) {
            return String.init(format: "%0.0f min", interval / 60)
        } else {
            return String.init(format: "%0.0f sec", interval)
        }
    }
}
